package com.chinaitop.depot.grainAnalysis.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;

import com.chinaitop.depot.grainAnalysis.mapper.StorageExceptionAndOnforeMapper;
import com.chinaitop.depot.grainAnalysis.mapper.StorageGrainAnalysisMapper;
import com.chinaitop.depot.grainAnalysis.model.StorageExceptionAndOnfore;
import com.chinaitop.depot.grainAnalysis.model.StorageExceptionAndOnforeExample;
import com.chinaitop.depot.grainAnalysis.model.StorageGrainAnalysis;
import com.chinaitop.depot.grainAnalysis.model.StorageGrainAnalysisExample;
import com.chinaitop.depot.grainAnalysis.service.StorageGrainAnalysisService;
import com.chinaitop.utils.ParameterUtil;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

@Service
public class StorageGrainAnalysisServiceImpl implements StorageGrainAnalysisService {

	@Resource
	private StorageGrainAnalysisMapper analysisMapper;

	@Resource
	private StorageExceptionAndOnforeMapper exceptionAndOnforeMapper;

	@Override
	public void add(StorageGrainAnalysis storageGrainAnalysis) {
		analysisMapper.insertSelective(storageGrainAnalysis);
	}

	@Override
	public void remove(Integer id) {
		analysisMapper.deleteByPrimaryKey(id);
	}

	@Override
	public void update(StorageGrainAnalysis storageGrainAnalysis) {
		analysisMapper.updateByPrimaryKeySelective(storageGrainAnalysis);
	}

	@Override
	public StorageGrainAnalysis findByParmaryKey(Integer id) {
		return analysisMapper.selectByPrimaryKey(id);
	}

	@Override
	public List<StorageGrainAnalysis> findByCondition(StorageGrainAnalysisExample example) {
		return analysisMapper.selectByExample(example);
	}

	@Override
	public Map<String, Object> findByIdAnalysisObj(Integer id, Integer analysisType, Map<String, Object> map) {

		// 查询主表数据
		StorageGrainAnalysis grain = analysisMapper.selectByPrimaryKey(id);
		map.put("grain", grain);

		// 查询三温检查数据
		Integer houseId = grain.getStorehouseId();
		Integer wareId = grain.getWarehouseId();
		if (analysisType == 0) {// 周分析
			map.put("houseId",grain.getStorehouseId());
			map.put("wareId",grain.getWarehouseId());
			map.put("orgId",grain.getOrgId());
			map.put("startDate",ParameterUtil.date2string(grain.getCheckDateStart()));
			map.put("endDate",ParameterUtil.date2string(grain.getCheckDateEnd()));
		} else if (analysisType == 1) {// 月分析
			getMonthThreeTempDate(houseId, wareId, grain.getYear(), grain.getMonth(),
					grain.getOrgId(), map);
		} else if (analysisType == 2) {//年分析
			getYearThreeTempDate(houseId, wareId, grain.getYear(), grain.getOrgId(), map);
		}
		
		
		// 查询粮情异常和四无情况数据
		getExceptionAndOnfore(grain.getAnalysisId(), map);

		return map;
	}

	@Override
	public Map<String, Object> getMonthThreeTempDate(Integer houseId, Integer wareId, Integer year, Integer month,
			Integer orgId, Map<String, Object> map) {
		/**
		 * 思路：1、将当前月的所有周分析数据拿到；
		 * 2、然后按照每条周分析数据的检查时间范围来查询每周对应的三温检查数据；
		 * 3、将每周的三温检查数据取到第一条作为一周的代表数据；
		 * 4、将每周的三温检查代表数据放入集合中返回
		 */

		List list = new ArrayList();

		//将当前月的所有周分析数据拿到；
		StorageGrainAnalysisExample grainExample = new StorageGrainAnalysisExample();
		StorageGrainAnalysisExample.Criteria grainCriteria = grainExample.createCriteria();
		grainCriteria.andYearEqualTo(year);
		if (month.toString().length() == 1) {
			month = Integer.parseInt("0" + month.toString());
		}
		grainCriteria.andMonthEqualTo(month);
		grainCriteria.andStorehouseIdEqualTo(houseId);
		grainCriteria.andWarehouseIdEqualTo(wareId);
		grainCriteria.andAnalysisTypeEqualTo(0);
		grainExample.setOrderByClause("id asc");
		List<StorageGrainAnalysis> grainList = analysisMapper.selectByExample(grainExample);

		Map<String, Object> param_map = new HashMap<String, Object>();
		param_map.put("house_id", houseId);
		Map<String, Object> house_map = analysisMapper.selectStoreHouseObj(param_map);
		String houseCode = house_map.get("storehouse_code")+"";
		
		//然后按照每条周分析数据的检查时间范围来查询每周对应的三温检查数据,每周的取一条数据组成集合返回；
		if (null != grainList && grainList.size() > 0) {
			for (StorageGrainAnalysis grain : grainList) {
				Date startDate = grain.getCheckDateStart();
				String startDate_str = ParameterUtil.date2string(startDate);

				Date endDate = grain.getCheckDateEnd();
				String endDate_str = ParameterUtil.date2string(endDate);

				Map<String, Object> dateMap = new HashMap<>();
				dateMap.put("houseId",houseCode);
				dateMap.put("startDate",startDate_str);
				dateMap.put("endDate",endDate_str);
				dateMap.put("orgId",orgId);
				list.add(dateMap);
			}
			
			if (null == list || list.size() == 0) {
				list = new ArrayList();
			}
		} else {
			list = new ArrayList();
		}
		map.put("threeTempCheckList", list);

		return map;
	}
	
	@Override
	public Map<String, Object> getYearThreeTempDate(Integer houseId, Integer wareId, Integer year, Integer orgId,
			Map<String, Object> map) {
		/**
		 * 思路：1、将当前年份的所有月分析数据拿到；
		 * 2、将每月的三温检查数据取到第一条作为一月的代表数据；
		 * 3、将每月的三温检查代表数据放入集合中返回
		 */

		List list = new ArrayList();
		
		//将当前月的所有周分析数据拿到；
		StorageGrainAnalysisExample grainExample = new StorageGrainAnalysisExample();
		StorageGrainAnalysisExample.Criteria grainCriteria = grainExample.createCriteria();
		grainCriteria.andYearEqualTo(year);
		grainCriteria.andStorehouseIdEqualTo(houseId);
		grainCriteria.andWarehouseIdEqualTo(wareId);
		grainCriteria.andAnalysisTypeEqualTo(1);
		grainExample.setOrderByClause("id asc");
		List<StorageGrainAnalysis> grainYearList = analysisMapper.selectByExample(grainExample);
		
		//然后按照每条周分析数据的检查时间范围来查询每周对应的三温检查数据,每周的取一条数据组成集合返回；
		if (null != grainYearList && grainYearList.size() > 0) {
			list = new ArrayList();
			Integer month = null;
			Map<String, Object> monthMap = new HashMap<>();
			for (StorageGrainAnalysis grain : grainYearList) {

				//获取每个月的月分析数据
//				monthMap = getMonthThreeTempDate(houseId, wareId, year, month, grain.getOrgId(), monthMap);
				
				//取月数据的第一条为代当前月的代表数据，最后结果是有几个月的分析数据就返回几条数据
//				StorageThreetempcheck threetempcheck = ((List<StorageThreetempcheck>) monthMap.get("threeTempCheckList")).get(0);
				list.add(grain.getMonth());
			}
			
			if (null == list || list.size() == 0) {
				list = new ArrayList();
			}
		} else {
			list = new ArrayList();
		}
		map.put("threeTempCheckList", list);

		return map;
	}
	
	private Map<String, Object> getExceptionAndOnfore(String analysisId, Map<String, Object> map) {
		StorageExceptionAndOnforeExample example = new StorageExceptionAndOnforeExample();
		StorageExceptionAndOnforeExample.Criteria criteria = example.createCriteria();
		criteria.andAnalysisIdEqualTo(analysisId);
		example.setOrderByClause("id asc");

		List<StorageExceptionAndOnfore> list = exceptionAndOnforeMapper.selectByExample(example);

		map.put("exceptionAndOnforeList", list);

		return map;
	}

	@Override
	public Map<String, Object> updateGrainAndException(String grainJson,String exceptionJson,
			Map<String, Object> map, Integer type,Integer orgId, Integer userId) {

		ObjectMapper mappers = new ObjectMapper();
		try {
			StorageGrainAnalysis grain = mappers.readValue(grainJson, StorageGrainAnalysis.class);
			if (grain.getId() == null || "".equals(grain.getId())) {

				if (type == 0) {
					Date startDate = grain.getCheckDateStart();
					String startDate_str = ParameterUtil.date2string(startDate);
					String year = startDate_str.substring(0, 4);
			        String month = startDate_str.substring(5, 7);
			        grain.setYear(Integer.parseInt(year));
					grain.setMonth(Integer.parseInt(month));
				}

				grain.setOrgId(orgId);
				grain.setInputDate(new Date());
				grain.setAuditorStatus(0); // 新数据设置为未审核状态
				grain.setAnalysisType(type); //分析类型：0周分析，1月分析，2年分析
				analysisMapper.insertSelective(grain);
			} else {
				analysisMapper.updateByPrimaryKeySelective(grain);
			}

			//更新粮食异常和四无分析信息
			List<StorageExceptionAndOnfore> exceptionList = mappers.readValue(exceptionJson,
					new TypeReference<List<StorageExceptionAndOnfore>>() {
					});
			editException(exceptionList, grain.getAnalysisId(), userId);
			map.put("status", "success");
			map.put("msg", "提交成功！");
		} catch (Exception e) {
			e.printStackTrace();
		}
		return map;
	}

	/**
	 * 更新粮食异常和四无分析信息
	 * 
	 * @param edcationList
	 * @param analysisId
	 * @param userId
	 */
	private void editException(List<StorageExceptionAndOnfore> edcationList, String analysisId, Integer userId) {
		for (StorageExceptionAndOnfore obj : edcationList) {
			if (null == obj.getId() || "".equals(obj.getId())) {
				obj.setAnalysisId(analysisId);
				obj.setInputPerson(userId);
				exceptionAndOnforeMapper.insertSelective(obj);
			} else {
				exceptionAndOnforeMapper.updateByPrimaryKeySelective(obj);
			}
		}
	}

}
